#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
	Slixmpp: The Slick XMPP Library
	Copyright (C) 2010  Nathanael C. Fritz
	This file is part of Slixmpp.

	See the file LICENSE for copying permission.
"""

# Code source: https://git.poez.io/slixmpp/tree/examples/muc.py

import logging
from getpass import getpass
from argparse import ArgumentParser

import slixmpp
import ssl, os, requests, urllib

import subprocess

class MUCBot(slixmpp.ClientXMPP):

	"""
	A simple Slixmpp bot that will execute 
	command line programmes and publish 
	their output to a html page. And it says 
	hello to anyone that enters the room.
	"""

	def __init__(self, jid, password, room, nick):
		slixmpp.ClientXMPP.__init__(self, jid, password)

		self.room = room
		self.nick = nick

		# The session_start event will be triggered when
		# the bot establishes its connection with the server
		# and the XML streams are ready for use. We want to
		# listen for this event so that we we can initialize
		# our roster.
		self.add_event_handler("session_start", self.start)

		# The groupchat_message event is triggered whenever a message
		# stanza is received from any chat room. If you also also
		# register a handler for the 'message' event, MUC messages
		# will be processed by both handlers.
		self.add_event_handler("groupchat_message", self.muc_message)

		# The groupchat_presence event is triggered whenever a
		# presence stanza is received from any chat room, including
		# any presences you send yourself. To limit event handling
		# to a single room, use the events muc::room@server::presence,
		# muc::room@server::got_online, or muc::room@server::got_offline.
		self.add_event_handler("muc::{}::got_online".format(self.room), self.muc_online)


	def start(self, event):
		"""
		Process the session_start event.

		Typical actions for the session_start event are
		requesting the roster and broadcasting an initial
		presence stanza.
		"""
		self.get_roster()
		self.send_presence()

		# https://xmpp.org/extensions/xep-0045.html
		self.plugin['xep_0045'].join_muc(self.room,
										 self.nick,
										 # If a room password is needed, use:
										 # password=the_room_password,
										 wait=True)

	def muc_message(self, msg):
		"""
		Process incoming message stanzas from any chat room. Be aware
		that if you also have any handlers for the 'message' event,
		message stanzas may be processed by both handlers, so check
		the 'type' attribute when using a 'message' event handler.

		Whenever the bot's nickname is mentioned, respond to
		the message.

		IMPORTANT: Always check that a message is not from yourself,
				   otherwise you will create an infinite loop responding
				   to your own messages.

		This handler will reply to messages that mention
		the bot's nickname.

		Arguments:
			msg -- The received message stanza. See the documentation
				   for stanza objects and the Message stanza to see
				   how it may be used.
		"""

		# some inspection commands
		# print('\nMessage:{}'.format(msg))
		# print('\nMessage TYPE:{}'.format(msg['type']))
		# print('\nMessage body:{}'.format(msg['body']))
		# print('\nMessage OOB:{}'.format(msg['oob']))
		# print('\nMessage OOB URL:{}'.format(msg['oob']['url']))
		# print('\nMessage MUCK NICK:{}'.format(msg['mucnick']))

		# Always check that a message is not the bot itself, otherwise you will create an infinite loop responding to your own messages.
		if msg['mucnick'] != self.nick:

			if '#publish' in msg['body']:

				options = '''
#traceroute [URL]
#whois [URL]
#who
'''

				# Send a reply
				self.send_message(mto=msg['from'].bare,
								  mbody="What would you like to do?\n {}.".format(options),
								  mtype='groupchat')

			if '#traceroute' in msg['body']:
				url = msg['body'].replace('#traceroute', '').strip()
				output = subprocess.check_output(['traceroute', url])
				print(output)

			if '#whois' in msg['body']:
				url = msg['body'].replace('#whois', '').strip()
				output = subprocess.check_output(['whois', url])
				print(output)

			if '#who' in msg['body']:
				output = subprocess.check_output(['who'])
				print(output)
			
			if output:
				for line in output.decode().split('\n'):
					self.send_message(mto=msg['from'].bare,
							  mbody=line,
							  mtype='groupchat')

	def muc_online(self, presence):
		"""
		Process a presence stanza from a chat room. In this case,
		presences from users that have just come online are
		handled by sending a welcome message that includes
		the user's nickname and role in the room.

		Arguments:
			presence -- The received presence stanza. See the
						documentation for the Presence stanza
						to see how else it may be used.
		"""
		if presence['muc']['nick'] != self.nick and presence['muc']['nick'] != '':
			self.send_message(mto=presence['from'].bare,
							  mbody="Hello, {} {}. If you want to publish something, please write #publish and see what options you have.".format(presence['muc']['role'], presence['muc']['nick']),
							  mtype='groupchat')


if __name__ == '__main__':
	# Setup the command line arguments.
	parser = ArgumentParser()

	# output verbosity options.
	parser.add_argument("-q", "--quiet", help="set logging to ERROR",
						action="store_const", dest="loglevel",
						const=logging.ERROR, default=logging.INFO)
	parser.add_argument("-d", "--debug", help="set logging to DEBUG",
						action="store_const", dest="loglevel",
						const=logging.DEBUG, default=logging.INFO)

	# JID and password options.
	parser.add_argument("-j", "--jid", dest="jid",
						help="JID to use")
	parser.add_argument("-p", "--password", dest="password",
						help="password to use")
	parser.add_argument("-r", "--room", dest="room",
						help="MUC room to join")
	parser.add_argument("-n", "--nick", dest="nick",
						help="MUC nickname")

	args = parser.parse_args()

	# Setup logging.
	logging.basicConfig(level=args.loglevel,
						format='%(levelname)-8s %(message)s')

	if args.jid is None:
		args.jid = input("User: ")
	if args.password is None:
		args.password = getpass("Password: ")
	if args.room is None:
		args.room = input("MUC room: ")
	if args.nick is None:
		args.nick = input("MUC nickname: ")

	# Setup the MUCBot and register plugins. Note that while plugins may
	# have interdependencies, the order in which you register them does
	# not matter.
	xmpp = MUCBot(args.jid, args.password, args.room, args.nick)
	xmpp.register_plugin('xep_0030') # Service Discovery
	xmpp.register_plugin('xep_0045') # Multi-User Chat
	xmpp.register_plugin('xep_0199') # XMPP Ping

	# Connect to the XMPP server and start processing XMPP stanzas.
	xmpp.connect()
	xmpp.process()

